#!/usr/bin/env python 

# Copyright (C) 2006 Benjamin Mako Hill <mako@atdot.cc>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301
# USA

__author__      = "Benjamin Mako Hill"
__version__     = "0.2"
__url__         = "http://mako.cc/projects/pyblosxom"
__description__ = "Run comments through Akismet."

import sys
import os
import os.path
import re
from getopt import gnu_getopt

# insert necessary pyblosxom module directories and the config directory
# in thte path 
sys.path.insert(1, os.path.dirname(sys.path[0]))
from config import py as cfg
sys.path.pop(1)

# set some variables based on the configuration file
api_key = cfg['akismet_api_key']
base_url = cfg['base_url']

# import the akismet module
for path in cfg['plugin_dirs']:
    sys.path.insert(1, os.path.dirname(sys.path[0]) + "/" + path)
from akismet import Akismet

# initialize the akismet module
api = Akismet(api_key, base_url, agent='PyBlosxom/1.3')
if not api.verify_key():
    print >>sys.stderr, "Could not verify akismet API key.",
    sys.exit()

# define the main subroutine to be run once per file submitted
def submit_spam_comment(comment=""):
    match = re.match(r'^.*<item>(.*)</item>.*$(?s)', comment)
    if match:
        comment = match.group(1).strip()
    else:
        return False

    fields = { 'description' : 'comment',
               'email' : 'comment_author_email',
               'author' : 'comment_author',
               'ipaddress' : 'user_ip',
               'link' : 'comment_author_url' }

    data = {}
    for field in fields:
        match = re.search((r'<%s>(.*)</%s>(?s)' % (field, field)), comment)
        if match:
            # go through and make sure that the file decodes properly
            # while things are going
            data[fields[field]] = ""
            for char in list(match.group(1)):
                try:
                    char.encode('ascii')
                except:
                    data[fields[field]] = data[fields[field]] + \
                                          "&#" + str(ord(char)) + ";"
                else:
                    data[fields[field]] = data[fields[field]] + char
            
    if data.has_key('comment') and not data['comment']:
        return False
    else:
        body = data['comment']

    # blank user_agent and user_ip because it's required
    data['user_agent'] = ""

    global api
    try:
        api.submit_spam(body, data, False)
        return True
    except:
        print >>sys.stderr, "Comment *not* subumitted successfully!"
        raise


## run once per file submitted
##################################
# set default options
delete_files = False

# extract options
options, args = gnu_getopt(sys.argv[1:], 'd')
for option, argument in options:
    if option == "-d":
        delete_files = True

if len(args) > 0:
    for arg in args: 
        if not os.path.exists(arg):
            print "File does not exist: %s" % arg
            continue 
        comment = open(arg, 'r').read()

        # submit the comment as spam
        rv = submit_spam_comment(comment)
        if not rv:
            print "Could not parse comment file: %s" % arg
            continue
        
        # delete the file if the option has been passed
        if delete_files:
            os.unlink(arg)
            print "Comment in file submitted as spam and deleted: %s" % arg
        else:
            print "Comment in file submitted as spam: %s" % arg
else:
    print >>sys.stderr, """USAGE:
  akismet_spam_submit.py [-d] filename1 [filename2 ...]

You must enter at least one filename. Use \"-d\" to delete files after
submission."""
